/*
 * Final_Project.c
 *
 * Created: 4/4/2012 8:43:40 PM
 *  Author: Mark and Doug (MaD)
 */ 

/*
 * Includes
 *****************************************/
#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "usart_driver.h"
#include "avr_compiler.h"
#include "dfll_trim.h"
#include "board.h"
#include "usart_dma_test.h"
#include "dma_driver.h"
#include "dac_driver.h"
#include "wifly.h"
#include "data_handling.h"
#include "Final_Project.h"

#define TIME_TO_REQUEST_MORE_DATA 11
#define PACKET_SIZE 512
//Packetsize/DMA_NUM_BURSTS_IN_BLOCK
//	1024/32 = 32
//	512/32 = 16
//	256/32 = 8
//	128/32 = 4
#define NUM_DMA_TRANS_THRESH 16

//Define length of streaming buffer
#define RX_BYTE_BUFFER_LEN 4
#define RX_WORD_BUFFER_LEN (RX_BYTE_BUFFER_LEN >> 1)

#ifdef STEREO
#define DATA_STREAM_CUTOFF 1500
#define HIGH_PACKET_REF	2816
#define LOW_PACKET_REF 1300
#define NORM_PACKET_REF 2048
#else
#define DATA_STREAM_CUTOFF 1500
#define HIGH_PACKET_REF	4000
#define LOW_PACKET_REF 1300
#define NORM_PACKET_REF 4000 
#endif //STEREO

char wifly_data_packet2[2] = {'M', 'G'};

/*
 * Function defines
 *****************************************/
void initialize(void);
void setup_dac(void);
void Begin_Stream(void);
void setup_debug(void);
void clear_rx_buf(void);
void enable_audio(void);
void disable_audio(void);

//Initialize rx buffer type
union rx_buffer_t{
	uint16_t word[RX_WORD_BUFFER_LEN];
	uint8_t byte[RX_BYTE_BUFFER_LEN];
};

//Flag to tell Wifly to send packet
volatile uint8_t send_packet = 1;

//Number of overflows = number of samples handled by DAC since last DMA setup
volatile uint8_t num_overflows_since_starting_DMA = DMA_NUM_BURSTS_IN_BLOCK;

volatile uint16_t timer1 = 0;
volatile uint16_t timer2 = 0;

volatile uint8_t num_dma_transfer = 0;
volatile uint8_t first_time = 1;
volatile uint8_t no_delay = 0;

uint8_t audio_enabled = 0;

/*
 * main
 *****************************************/
int main(void)
{
	
    initialize();
	clear_rx_buf();
	Begin_Stream();
	
}

/*
 * All the setup stuff goes here
 *****************************************/
void initialize(void)
{	
	//set the frequency to our custom frequency using the dfll
	//to trim the clock
	trim_to_custom_freq();
	
	setup_debug();
	
	setup_dac();
	
	SetupWiflyUsartInterrupts9600();
	//SetupUsartInterrupts921600();
	
	//Use interrupts from now on
	sei();
	
	initialize_wifly_intr();
	cli();
		
}

void setup_debug(void) 
{
	//setup LED direction
	// Initialize LEDs
    LEDPORT.DIR = 0xFF;
    LEDPORT.OUT = 0x00; //LED OFF
}

void setup_dac(void)
{
	// First we have to enable the audio amplifier by setting PQ2 + PQ3 high.
	PORTB.PIN2CTRL = (PORTB.PIN2CTRL & ~PORT_OPC_gm) | PORT_OPC_PULLUP_gc;
	PORTB.PIN3CTRL = (PORTB.PIN3CTRL & ~PORT_OPC_gm) | PORT_OPC_PULLUP_gc;

    // Enable overflow interrupt
    TCC0.INTCTRLA = ( TCC0.INTCTRLA & TC0_OVFINTLVL_gm ) | TC_OVFINTLVL_HI_gc;
    TCD0.INTCTRLA = ( TCD0.INTCTRLA & TC0_OVFINTLVL_gm ) | TC_OVFINTLVL_LO_gc;
	
	// Enable medium interrupt level in PMIC and enable global interrupts.
	PMIC.CTRL |= PMIC_LOLVLEN_bm | PMIC_HILVLEN_bm;
    
//     // Enable Timer C0, prescaler div1 means Main Clock
//     TCC0.CTRLA = ( TCC0.CTRLA & ~TC0_CLKSEL_gm ) | TC_CLKSEL_DIV1_gc;
    // Enable Timer D0, prescaler div1 means Main Clock
	TCD0.CTRLA = ( TCD0.CTRLA & ~TC0_CLKSEL_gm ) | TC_CLKSEL_DIV1_gc;
	
	//start timer D0, not C0
	//33868800
	TCD0.PER = 33869;
	TCC0.PER = TIMER_CD0_PERIOD;
	
	//setup DACB in dual channel mode
	DACB.CTRLB = DAC_CHSEL_DUAL_gc | DAC_CH1TRIG_bm | DAC_CH0TRIG_bm;
	DACB.CTRLC = DAC_REFSEL_AVCC_gc | DAC_LEFTADJ_bm ;
	//DACB.TIMCTRL = (uint8_t) sampleInterval | refreshInterval;
	DACB.CTRLA |= DAC_CH1EN_bm | DAC_CH0EN_bm | DAC_ENABLE_bm;
	DACB.EVCTRL = 8;
	
	// Source for event channel 0 and 1 is TCC0 overflow
	EVSYS.CH0MUX = 0b11000000;
	EVSYS.CH1MUX = 0b11000000;
}

void Begin_Stream(void) {
	
	SetupReceiveChannel();
	
	no_delay = 1;
	
	//Signal setup is complete
	BOTH_ON();
	_delay_ms(500);
	BOTH_OFF();
	
	sei();
	
	while(1){		
		if (audio_enabled == 1){
			//send packet when we want to slow data sending rate
			
			if (timer1 >= 80){
 				if ((bytes_in_left_audio_buffer() >= HIGH_PACKET_REF)){		
					wifly_send_request_stream_slow();
					timer1 = 0;
 				} else if ((bytes_in_left_audio_buffer() <= LOW_PACKET_REF)){
					wifly_send_request_stream_fast();
					timer1 = 0;
				}
			}						
			//Reset audio streaming when song ends
			if (timer2 > 2000){
				disable_audio();
			}				
		}		 	 
	
		
		//check for complete dma transfer
		if (dma_rx_buffer_ready == true)
		{
			dma_rx_buffer_ready = false;
			// copy dma buffer to audio buffer
			if (dma_rx_buffer_number == 0) {
				//buffer2 just finished
#ifdef NO_ZERO
	#ifdef STEREO
				if (copy_8bit_stereo_to_buffer_no_zero(dma_rx_buffer2,DMA_RX_BUFFER_LEN) == -1) {YELLOW_TOGGLE();}
	#else
				if (copy_8bit_mono_to_buffer_no_zero(dma_rx_buffer2,DMA_RX_BUFFER_LEN) == -1) {YELLOW_TOGGLE();}
	#endif //STEREO
#else
				if (copy_8bit_mono_to_buffer(dma_rx_buffer2,DMA_RX_BUFFER_LEN) == -1) {/*YELLOW_TOGGLE();*/}
#endif	//NO_ZERO
				
			} else if (dma_rx_buffer_number == 1)  {
				//buffer0 just finished
#ifdef NO_ZERO
	#ifdef STEREO
				if (copy_8bit_stereo_to_buffer_no_zero(dma_rx_buffer0,DMA_RX_BUFFER_LEN) == -1) {YELLOW_TOGGLE();}
	#else
				if (copy_8bit_mono_to_buffer_no_zero(dma_rx_buffer0,DMA_RX_BUFFER_LEN) == -1) {YELLOW_TOGGLE();}
	#endif //STEREO
#else
				if (copy_8bit_mono_to_buffer(dma_rx_buffer0,DMA_RX_BUFFER_LEN) == -1) {/*YELLOW_TOGGLE();*/}
#endif	//NO_ZERO
			} else {
				//buffer1 just finished
#ifdef NO_ZERO
	#ifdef STEREO
				if (copy_8bit_stereo_to_buffer_no_zero(dma_rx_buffer1,DMA_RX_BUFFER_LEN) == -1) {YELLOW_TOGGLE();}
	#else
				if (copy_8bit_mono_to_buffer_no_zero(dma_rx_buffer1,DMA_RX_BUFFER_LEN) == -1) {YELLOW_TOGGLE();}
	#endif //SETERO
#else
				if (copy_8bit_mono_to_buffer(dma_rx_buffer1,DMA_RX_BUFFER_LEN) == -1) {/*YELLOW_TOGGLE();*/}
#endif	//NO_ZERO
			}		
		}
		
		//enable audio when buffer size reaches threshold
		if ((bytes_in_left_audio_buffer() > DATA_STREAM_CUTOFF) && (audio_enabled == 0)){
			enable_audio();
			audio_enabled = 1;
			
		} else if (audio_enabled == 0 && timer1 >= 20) {
			//send packet to begin process
			//wifly_send_request_stream_normal();
			timer1 = 0;
		}
	}		
}

//take all leftover data from USART and put it in same place in same place
void clear_rx_buf(void){
	
	union rx_buffer_t trash_Buf;
	uint8_t i = 0;
	
	while (USART_RXBufferData_Available(&USART_data)) 
	{
		trash_Buf.byte[i] = USART_RXBuffer_GetByte(&USART_data);
	}		
}

//Enable timer interrupts to begin audio streaming
void enable_audio(void){
	cli();
	// Enable Timer C0, prescaler div1 means Main Clock
	TCC0.CTRLA = ( TCC0.CTRLA & ~TC0_CLKSEL_gm ) | TC_CLKSEL_DIV1_gc;
	// Enable Timer D0, prescaler div1 means Main Clock
    TCD0.CTRLA = ( TCD0.CTRLA & ~TC0_CLKSEL_gm ) | TC_CLKSEL_DIV1_gc;
	sei();
}

//Enable timer interrupts to begin audio streaming
void disable_audio(void){
	// disable
	cli();
	TCC0.CTRLA = 0;
	sei();
	
	BOTH_OFF();
	audio_enabled = 0;
	first_time = 1;
}

//  Receive complete interrupt service routine.
//  Calls the common receive complete handler with pointer to the correct USART
//  as argument.
 
ISR( USART_RXC_vect ) // Note that this vector name is a define mapped to the correct interrupt vector
{                     // See "board.h"
	USART_RXComplete( &USART_data );
}

//  Data register empty  interrupt service routine.
//  Calls the common data register empty complete handler with pointer to the
//  correct USART as argument.

ISR( USART_DRE_vect ) // Note that this vector name is a define mapped to the correct interrupt vector
{                     // See "board.h"
    USART_DataRegEmpty( &USART_data );
	if (no_delay == 0){
		delay_us(20);
	}
	
}


