/*
 * wifly.c
 *
 * Created: 4/13/2012 2:24:20 PM
 *  Author: Doug
 */ 

#include "wifly.h"
#include "dma_driver.h"
#include "board.h"
#include "usart_dma_test.h"

#define AD_HOC
#ifndef AD_HOC
#define REDROVER
#endif //AD_HOC


#define RX_BUF_LEN  400
char Rx_Buf[RX_BUF_LEN];

/*
 * String Initializations
 *****************************************/
// Buffers to send/receive
#define CMD_RESP_LEN 3
char wifly_cmd_resp[CMD_RESP_LEN] = "CMD";
#define AOK_RESP_LEN 3
char wifly_aok_resp[AOK_RESP_LEN] = "AOK";
#define READY_RESP_LEN 5
char wifly_ready_resp[READY_RESP_LEN] = "READY";

//enter command mode
#define WIFLY_ENTER_CMD_MODE_LEN 3
char wifly_enter_cmd_mode[WIFLY_ENTER_CMD_MODE_LEN] = "$$$";

//Sequence to join network
#define WIFLY_SET_SSID_LEN 14 
char wifly_set_ssid[WIFLY_SET_SSID_LEN] = "set wlan ssid ";// <string>" // set the network name
#define WIFLY_SET_PASS_LEN 16 
char wifly_set_pass[] = "set wlan phrase ";// <string>" // set the passphrase for WPA and WPA2 modes

//enter adhoc
#define L1 16
char string_1[L1] = "set wlan join 4\r";
#define L2 32
char string_2[L2] = "set wlan ssid MAD_adhoc_network\r";
#define L3 16
char string_3[L3] = "set wlan chan 4\r";
#define L4 27
char string_4[L4] = "set ip address 169.254.1.1\r";
#define L5 27
char string_5[L5] = "set ip netmask 255.255.0.0\r";
#define L6 14
char string_6[L6] = "set ip dhcp 0\r";

//Set up the protocol and port number
#define WIFLY_SET_UDP_MODE_LEN 15
char wifly_set_udp_mode[WIFLY_SET_UDP_MODE_LEN] = "set ip proto 1\r"; // enable UDP as the protocol


//Mark: 169.254.60.45
//Doug: 169.254.89.52
#ifdef AD_HOC
#define WIFLY_SET_REM_IP_LEN 26
char wifly_set_rem_ip[WIFLY_SET_REM_IP_LEN] = "set ip host 169.254.89.52\r"; // set the IP address of remote host 
#endif //AD_HOC

//Mark RedRover: 128.84.123.177
//Doug RedRover: 128.253.195.69
#ifdef REDROVER
#define WIFLY_SET_REM_IP_LEN 27
char wifly_set_rem_ip[WIFLY_SET_REM_IP_LEN] = "set ip host 128.84.123.143\r"; // set the IP address of remote host 
#endif //REDROVER


#define WIFLY_SET_IP_REM_PORT_LEN 20
char wifly_set_ip_rem_port[WIFLY_SET_IP_REM_PORT_LEN] = "set ip remote 30291\r"; // set the remote port number on which the host is listening
#define WIFLY_SET_IP_LOC_PORT_LEN 18
char wifly_set_ip_loc_port[WIFLY_SET_IP_LOC_PORT_LEN] = "set ip local 5033\r"; // set the port number on which the WiFly module will listen
#define WIFLY_SAVE_LEN 5
char wifly_save[WIFLY_SAVE_LEN] = "save\r"; // saves the settings in "cust" config file
#define WIFLY_REBOOT_LEN 7
char wifly_reboot[WIFLY_REBOOT_LEN] = "reboot\r"; // reboots the module so that the above settings take effect
#define WIFLY_EXIT_LEN 5
char wifly_exit[WIFLY_EXIT_LEN] = "exit\r"; // exits command mode
#define WIFLY_EXIT_RESP_LEN 4
char wifly_exit_resp[WIFLY_EXIT_RESP_LEN] = "EXIT"; // exits response mode

//Edit Broadcast packets sent
#define WIFLY_BROAD_ADD_LEN 36
char wifly_broad_add[WIFLY_BROAD_ADD_LEN] = "set broadcast address 169.254.60.45\r"; // sets broadcast address
#define WIFLY_BROAD_PORT_LEN 25
char wifly_broad_port[WIFLY_BROAD_PORT_LEN] = "set broadcast port 30271\r"; // sets broadcast port
#define WIFLY_BROAD_INT_LEN 25
char wifly_broad_int[WIFLY_BROAD_INT_LEN] = "set broadcast interval 0\r"; // disables broadcasting


//Setup the baud rate
#define WIFLY_SET_UART_BAUD_9600_LEN 22
char wifly_set_uart_baud_9600[WIFLY_SET_UART_BAUD_9600_LEN] = "set uart instant 9600\r";
#define WIFLY_SET_UART_BAUD_115200_LEN 24
char wifly_set_uart_baud_115200[WIFLY_SET_UART_BAUD_115200_LEN] = "set uart instant 115200\r";
#define WIFLY_SET_UART_BAUD_230400_LEN 24
char wifly_set_uart_baud_230400[WIFLY_SET_UART_BAUD_230400_LEN] = "set uart instant 230400\r";
#define WIFLY_SET_UART_BAUD_460800_LEN 24
char wifly_set_uart_baud_460800[WIFLY_SET_UART_BAUD_460800_LEN] = "set uart instant 460800\r";
#define WIFLY_SET_UART_BAUD_921600_LEN 24
char wifly_set_uart_baud_921600[WIFLY_SET_UART_BAUD_921600_LEN] = "set uart instant 921600\r";
#define WIFLY_SET_UART_BAUD_921600__FOREVER_LEN 21
char wifly_set_uart_baud_921600_forever[WIFLY_SET_UART_BAUD_921600_LEN] = "set uart baud 921600\r";
#define WIFLY_TEST_RESP_LEN 16
char wifly_test_resp[WIFLY_TEST_RESP_LEN] = "Got your packet!";

//setup Flow control
#define WIFLY_COMM_TIME_LEN 16
char wifly_comm_time[WIFLY_COMM_TIME_LEN] = "set comm time 5\r";
#define WIFLY_COMM_MATCH_LEN 19
char wifly_comm_match[WIFLY_COMM_MATCH_LEN] = "set comm match 0xd\r";
#define WIFLY_COMM_SIZE_LEN 16
char wifly_comm_size[WIFLY_COMM_SIZE_LEN] = "set comm size 2\r";

//Strings for Wifly data packet
#define WIFLY_DATA_PACKET_LEN 1
char wifly_request_slow[WIFLY_COMM_MATCH_LEN] = {'S'};
char wifly_request_normal[WIFLY_COMM_MATCH_LEN] = {'N'};
char wifly_request_fast[WIFLY_COMM_MATCH_LEN] = {'F'};
char wifly_request[2] = {'M', 'D'};

//Factory Reset
#define WIFLY_RESET_LEN 14
char wifly_reset[WIFLY_RESET_LEN] = "factory RESET\r";

//Connect to RedRover
#define WIFLY_JOIN_RED_LEN 14
char wifly_join_red[WIFLY_JOIN_RED_LEN] = "join RedRover\r";
#define WIFLY_JOIN_RESP_LEN 11
char wifly_join_resp[WIFLY_JOIN_RESP_LEN] = "Associated!";

//initialize the wifly with commands sent via interrupts
int initialize_wifly_intr(void)
{

	//enter command mode
	if (wifly_send_cmd_mode_intr() < 0)
	{
		//ran out of buffer
		//YELLOW_ON();
		return -1;
	}
	
	//Reset Wifly module to factory defaults
	if (wifly_reset_intr() < 0)
	{
		//ran out of buffer
		//YELLOW_ON();
		return -1;
	}


	#ifdef AD_HOC
	//create adhoc network
	if (create_adhoc_network_intr() < 0 )
	{
		//ran out of buffer
		//YELLOW_ON();
		return -1;
	}
	#endif //AD_HOC
	
	
	//enter udp state
	if (create_udp_connection_intr() < 0 )
	{
		//ran out of buffer
		//YELLOW_ON();
		return -1;
	}
	
	if (setup_comm_setting() < 0) 
	{
		//ran out of buffer or ERR
		//YELLOW_ON();
		return -1;		
	}
	
	//save state
	if (wifly_send_string_intr_nr(wifly_save,WIFLY_SAVE_LEN) < 0 )
	{
		//ran out of buffer
		//YELLOW_ON();
		return -1;
	}
	
	//restart
	if (wifly_send_string_intr_nr(wifly_reboot,WIFLY_REBOOT_LEN) < 0 )
	{
		//ran out of buffer
		//YELLOW_ON();
		return -1;
	}
	
	//wait for "READY"
	wifly_receive_ack_intr(wifly_ready_resp,READY_RESP_LEN);
	
	_delay_ms(250);
	
	
	//enter command mode
 	if (wifly_send_cmd_mode_intr() < 0)
 	{
 		//ran out of buffer
 		//YELLOW_ON();
 		return -1;
 	}
	
	#ifdef REDROVER
	if(wifly_join_red_intr() < 0) 
	{
		//ran out of buffer
		//YELLOW_ON();
		return -1;
	}
	#endif //REDROVER
	
	//update our baud rate
	if (change_serial_to_921600_intr() < 0 )
	{
		//ran out of buffer
		//YELLOW_ON();
		return -1;
	}
	
	
	// exit
	if (wifly_send_string_intr(wifly_exit,WIFLY_EXIT_LEN,wifly_exit_resp,WIFLY_EXIT_RESP_LEN) < 0) {	
 		//ran out of buffer or ERR
 		//YELLOW_ON();
 		return -1;
 	}
	
	//enter command mode
 	if (wifly_send_cmd_mode_intr() < 0)
 	{
 		//ran out of buffer
 		//YELLOW_ON();
 		return -1;
 	}
	 
	// exit
	if (wifly_send_string_intr(wifly_exit,WIFLY_EXIT_LEN,wifly_exit_resp,WIFLY_EXIT_RESP_LEN) < 0) {	
 		//ran out of buffer or ERR
 		//YELLOW_ON();
 		return -1;
 	}
	 
	return 0;
}

//Send commands to set up the desired ad hoc network
int create_adhoc_network_intr(void)
{
	if (wifly_send_string_intr_aok(string_1,L1) < 0 ) return -1;
	if (wifly_send_string_intr_aok(string_2,L2) < 0 ) return -1;
	if (wifly_send_string_intr_aok(string_3,L3) < 0 ) return -1;
	if (wifly_send_string_intr_aok(string_4,L4) < 0 ) return -1;
	if (wifly_send_string_intr_aok(string_5,L5) < 0 ) return -1;
	if (wifly_send_string_intr_aok(string_6,L6) < 0 ) return -1;
	return 0;
}

//Send commands to Wifly to set up to receive UDP packets
int create_udp_connection_intr( void ) 
{
	if (wifly_send_string_intr_aok(wifly_set_udp_mode,WIFLY_SET_UDP_MODE_LEN) < 0 ) return -1;
	if (wifly_send_string_intr_aok(wifly_set_rem_ip,WIFLY_SET_REM_IP_LEN) < 0 ) return -1;
	if (wifly_send_string_intr_aok(wifly_set_ip_rem_port,WIFLY_SET_IP_REM_PORT_LEN) < 0 ) return -1;
	if (wifly_send_string_intr_aok(wifly_set_ip_loc_port,WIFLY_SET_IP_LOC_PORT_LEN) < 0 ) return -1;
	
	//Disable UDP broadcast messages
	if (wifly_send_string_intr_aok(wifly_broad_int,WIFLY_BROAD_INT_LEN) < 0 ) return -1;
	
	//Set broadcast to send to host
	//if (wifly_send_string_intr_aok(wifly_broad_add,WIFLY_BROAD_ADD_LEN) < 0 ) return -1;
	//if (wifly_send_string_intr_aok(wifly_broad_port,WIFLY_BROAD_PORT_LEN) < 0 ) return -1;
	return 0;
}

//Send commands to setup data setting
int setup_comm_setting (void)
{
	if (wifly_send_string_intr_aok(wifly_comm_time,WIFLY_COMM_TIME_LEN) < 0 ) return -1;
	//if (wifly_send_string_intr_aok(wifly_comm_match,WIFLY_COMM_MATCH_LEN) < 0 ) return -1;
	//if (wifly_send_string_intr_aok(wifly_comm_size,WIFLY_COMM_SIZE_LEN) < 0 ) return -1;
	return 0;
}

//Reset Wifly to factory defaults
int wifly_reset_intr(void)
{
	return wifly_send_string_intr_nr(wifly_reset,WIFLY_RESET_LEN);
}


//Connect to RedRover on Wifly
int wifly_join_red_intr(void)
{
	return wifly_send_string_intr(wifly_join_red,WIFLY_JOIN_RED_LEN,wifly_join_resp, WIFLY_JOIN_RESP_LEN);
}


void wifly_send_test_ack_intr(void)
{
	wifly_send_string_intr_nr(wifly_test_resp,WIFLY_TEST_RESP_LEN);
}


int change_serial_to_115200_intr( void ) 
{
	wifly_send_string_intr_nr(wifly_set_uart_baud_115200,WIFLY_SET_UART_BAUD_115200_LEN);
	
	
	//wait til TX is empty
 	while(!USART_IsTXDataRegisterEmpty(&USART_WIFLY) || !USART_TXBuffer_IsEmpty(&USART_data))
 	{
 		nop();
 	}
	
	SetupWiflyUsartInterrupts115200();
	
	//todo change this
	return 0;
}


int change_serial_to_230400_intr( void ) 
{
	wifly_send_string_intr_nr(wifly_set_uart_baud_230400,WIFLY_SET_UART_BAUD_230400_LEN);
	

	//wait til TX is empty
 	while(!USART_IsTXDataRegisterEmpty(&USART_WIFLY) || !USART_TXBuffer_IsEmpty(&USART_data))
 	{
 		nop();
 	}
	 
	SetupWiflyUsartInterrupts230400();
	
	//todo change this
	return 0;
}

int change_serial_to_460800_intr( void ) 
{
	wifly_send_string_intr_nr(wifly_set_uart_baud_460800,WIFLY_SET_UART_BAUD_460800_LEN);
	
	//wait til TX is empty
 	while(!USART_IsTXDataRegisterEmpty(&USART_WIFLY) || !USART_TXBuffer_IsEmpty(&USART_data))
 	{
 		nop();
 	}
	 
	SetupWiflyUsartInterrupts460800();
	
	//todo change this
	return 0;
}

int change_serial_to_921600_intr( void ) 
{

	wifly_send_string_intr_nr(wifly_set_uart_baud_921600,WIFLY_SET_UART_BAUD_921600_LEN);
	
	//wait til TX is empty
 	while(!USART_IsTXDataRegisterEmpty(&USART_WIFLY) || !USART_TXBuffer_IsEmpty(&USART_data))
 	{
 		nop();
 	}
	
	SetupWiflyUsartInterrupts921600();
	
	//todo change this
	return 0;
}

int change_serial_to_921600_intr_forever( void ) 
{

	wifly_send_string_intr_nr(wifly_set_uart_baud_921600_forever,WIFLY_SET_UART_BAUD_921600__FOREVER_LEN);
	
	//wait til TX is empty
 	while(!USART_IsTXDataRegisterEmpty(&USART_WIFLY) || !USART_TXBuffer_IsEmpty(&USART_data))
 	{
 		nop();
 	}
	
	//todo change this
	return 0;
}

int wifly_send_cmd_mode_intr(void)
{
	return wifly_send_string_intr(wifly_enter_cmd_mode,WIFLY_ENTER_CMD_MODE_LEN,wifly_cmd_resp,CMD_RESP_LEN);
}

int wifly_send_string_intr_aok(char * buffer, uint8_t len_1)
{
	return wifly_send_string_intr(buffer, len_1, wifly_aok_resp, AOK_RESP_LEN);
}

int wifly_send_string_intr(char * buffer, uint8_t len_1, char * resp, uint8_t len_2)
{
	wifly_send_string_intr_nr(buffer, len_1);
	
	return wifly_receive_ack_intr(resp, len_2);
}

int wifly_send_string_intr_nr(char * buffer, uint8_t len_1)
{
	// Send sendArray
	uint8_t i = 0;
	while (i < len_1) 
    {
		bool byteToBuffer;

		byteToBuffer = USART_TXBuffer_PutByte(&USART_data, buffer[i]);		
        if(byteToBuffer)
        {
			i++;
			
		}
	}
	
	return 0;
}

int wifly_receive_ack_intr(char * buffer, uint8_t len)
{
	// Fetch received data as it is received
	uint8_t i = 0;

    while (i < (RX_BUF_LEN - 2) ) 
    {
        if (USART_RXBufferData_Available(&USART_data)) 
        {
			Rx_Buf[i] = USART_RXBuffer_GetByte(&USART_data);
			i++;			
		}
		
		//if there are enough characters in the receive buffer
		if ( i >= len )
		{
			for (uint16_t i1 = 0; i1 <= i-len ; i1++)
			{
				for (uint16_t i2 = i1; i2 < i1 + len; i2++)
				{
					// Check that each element is received correctly
					if (i2 >= 2){
						if ((Rx_Buf[i2-2] == 'E') && (Rx_Buf[i2-1] =='R') && (Rx_Buf[i2] =='R')){
							return -2;
						}
					}
					
					if (Rx_Buf[i2] != buffer[i2-i1]) 
					{
						break;
					} else {
						//if this is the last char, the strings match at this position
						if (i2 == (i1 + len - 1))
						{
							return 0;
						}
					}
				}
			}
		}
	}
	return -1;
}	

//Send packet to ask for more data
void wifly_send_request_stream_fast(void){
	
	wifly_send_string_intr_nr(wifly_request_fast, WIFLY_DATA_PACKET_LEN);
	
}

//Send packet to ask for more data
void wifly_send_request_stream_normal(void){
	
	wifly_send_string_intr_nr(wifly_request_normal, WIFLY_DATA_PACKET_LEN);
	
}

//Send packet to ask for more data
void wifly_send_request_stream_slow(void){
	
	wifly_send_string_intr_nr(wifly_request_slow, WIFLY_DATA_PACKET_LEN);
	
}

//Send packet to ask for more data
void wifly_send_data_request(void){
	
	wifly_send_string_intr_nr(wifly_request, 2);
	
}