/*
	Sending and Receiving Packets Example (Simple version!)
	From "Networking for Game Programmers" - http://www.gaffer.org/networking-for-game-programmers
	Author: Glenn Fiedler <gaffer@gaffer.org>
	
	^^ THANKS GLENN!!
	to build, install mingw, navigate to this file and run 
	g++ Simple.cpp -o simple.exe -L. -lwsock32 -lsndfile-1 -Wall
	
	directory stuff courtesy of:
	http://www.informit.com/guides/content.aspx?g=cplusplus&seqNum=245
	vectors thanks to:
	http://www.mochima.com/tutorials/vectors.html
	numerous samples from:
	http://www.cplusplus.com
	
*/

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <windows.h>  
#include <errno.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>

#include "Net.h"
#include "sndfile.h"

#define REMOTE_IP_A 169
#define REMOTE_IP_B 254
#define REMOTE_IP_C 1
#define REMOTE_IP_D 1
#define REMOTE_PORT 5033
#define LOCAL_PORT	30291
#define FRAMES_IN_PACKET 512

#define VK_MEDIA_NEXT_TRACK 0xb0
#define VK_MEDIA_PREV_TRACK 0xb1
#define VK_MEDIA_PLAY_PAUSE 0xb3

#define TUNE_PERC 0.0000

//define feedback parameters
#ifdef STEREO
#define TUNE_MAX_DIF 0.01
#define TUNE_MIN_DIF 0.02
#define SLOW_MUL 1.5
#define FAST_MUL 0.05
#define INTERVAL_SUB 0.0001
#else
//0.01
//0.0025
#define TUNE_MAX_DIF 0.01
#define TUNE_MIN_DIF 0.0025
#define SLOW_MUL 1.5	//1.5
#define FAST_MUL 0.1
#define INTERVAL_SUB 0
#endif //STEREO

//Keyboard variables
char prev_space = 0;
char curr_space = 0;
char prev_n = 0;
char curr_n = 0;
char prev_p = 0;
char curr_p = 0;

double interval_norm;
double interval_slow;
double interval_fast;
double interval;
double dynamic_step;
double dynamic_max;
double dynamic_min;

int format = 0;
int channels = 0;
int frames_per_second = 0;
unsigned int current_song_index = 0;
int desired_num_reads = 0;

//Declaring the use of functions in standard classes
using std::cout;
using std::cin;
using std::endl;

using namespace std;
using namespace net;

bool pressedPlayPause(void);
bool pressedNext(void);
bool pressedPrevious(void);
bool is8BitMono(const char * path);
SNDFILE* getSoundfile(const char * path);
SNDFILE* playNextSong(vector<string> names);
void SetColor(int ForgC);

int main( int argc, char * argv[] )
{
	SetColor(10);

	printf("*****MAD-DOG STREAMER*****\n");
	printf("target: %d.%d.%d.%d:%d\n", REMOTE_IP_A, REMOTE_IP_B, REMOTE_IP_C, REMOTE_IP_D, REMOTE_PORT);
	printf("listening on %d\n", LOCAL_PORT);
		
	char song_dir[200];
	
	//take in and interpret the argument 
	if (argc > 1)
    {
		for (int count = 1; count < argc; count++)
		{
			strcpy(song_dir,argv[count]);
		}
    } else {
		strcpy(song_dir,"../");
	}
	printf("Looking for songs in: %s\n",song_dir);
	
	vector<string> names;
	//open directories
	DIR *pdir;
	struct dirent *pent;

	pdir=opendir(song_dir);
	if (!pdir){
		printf ("opendir() failure; terminating");
		exit(1);
	}
	errno=0;
	
	string wav (".wav");
	size_t found;
	
	while ((pent=readdir(pdir))){
		//check if it is a wav
		string s(pent->d_name);
		s= song_dir + s;
		found = s.find(wav);
		if (found!=string::npos)
			names.insert(names.end(), s);
	}
	
	if (errno){
		printf ("readdir() failure; terminating");
		exit(1);
	}
	closedir(pdir);
	
	//print dirs
	printf("Found the following wav files:\n");
	for (vector<string>::size_type i = 0; i < names.size(); i++)
		cout << names[i] + "\n"; 
	printf("\n");
	
	// initialize socket layer
	if ( !InitializeSockets() )
	{
		printf( "failed to initialize sockets\n" );
		exit(1);
	}

	Socket socket;
	if ( !socket.Open( LOCAL_PORT ) )
	{
		printf( "failed to create socket!\n" );
		exit(1);
	}
	//open sound file
	SNDFILE* sound_file = getSoundfile(names[0].c_str());
	printf("Playing: %s\n",names[0].c_str());
	if (sound_file == NULL)
	{
		printf("Error opening sound file\n");
		exit(1);
	}
	
	//start timing
	LARGE_INTEGER frequency;          // ticks per second
    LARGE_INTEGER t1, t2; // ticks
    double elapsedTime;
	
	if (!QueryPerformanceFrequency(&frequency))
	{
		printf("QueryPerformanceFrequency failed!\n");
		exit(1);
	}
	
	unsigned int count = 0;
	char paused = 0;
	int need_to_reload = 1;
	unsigned char mono_data[FRAMES_IN_PACKET];
	int audio_data[FRAMES_IN_PACKET*2];
	
	// start timer
    QueryPerformanceCounter(&t1);
	while ( 1 )
	{
	while ( 1 )
	{	//check button presses
		if (pressedPlayPause())
		{
			if (paused == 1) {
				paused = 0;
				printf("Play =)\n");
			} else {
				paused = 1;
				printf("Paused =|\n");
			}
		} else if (pressedNext()) {
			if (  sf_close(sound_file) != 0)
			{
				printf("problem closing the wave file.\n");
			}
	
			//play next song
			printf("Next: ");
			current_song_index++;
			if (current_song_index >= names.size())
				current_song_index = 0;

			sound_file = getSoundfile(names[current_song_index].c_str());
			printf("%s\n",names[current_song_index].c_str());
			if (sound_file == NULL)
			{
				printf("Error opening sound file\n");
				exit(1);
			}
		} else if (pressedPrevious()) {
			if (  sf_close(sound_file) != 0)
			{
				printf("problem closing the wave file.\n");
			}
			
			//play previous song
			printf("Previous: ");
			
			current_song_index--;
			if (current_song_index > names.size())
				current_song_index = names.size()-1;

			sound_file = getSoundfile(names[current_song_index].c_str());
			printf("%s\n",names[current_song_index].c_str());
			if (sound_file == NULL)
			{
				printf("Error opening sound file\n");
				exit(1);
			}
		}
		
		if (need_to_reload == 1)
		{
			need_to_reload = 0;
			int num_reads;
			
			//if this is not unsigned 8bit try to convert
			if ( ( format & SF_FORMAT_SUBMASK ) != SF_FORMAT_PCM_U8 ) {
				num_reads = sf_readf_int(sound_file, audio_data, FRAMES_IN_PACKET);
			} else {
				num_reads = sf_read_raw(sound_file, mono_data, FRAMES_IN_PACKET);
			}
			
			if (num_reads != desired_num_reads)
			{
				if (  sf_close(sound_file) != 0)
				{
					printf("problem closing the wave file.\n");
				}
				
				//play next song
				printf("Next: ");
				current_song_index++;
				if (current_song_index >= names.size())
					current_song_index = 0;

				sound_file = getSoundfile(names[current_song_index].c_str());
				printf("%s\n",names[current_song_index].c_str());
				if (sound_file == NULL)
				{
					printf("Error opening sound file\n");
					exit(1);
				}
				need_to_reload = 1;
				break;
			}
			
			if ( ( format & SF_FORMAT_SUBMASK ) != SF_FORMAT_PCM_U8 ) {
				if (channels == 2 && frames_per_second == 44100)
				{
					//this is signed so convert to unsigned 8 bit
					for (int i = 0; i < FRAMES_IN_PACKET ; i++)
					{
						mono_data[i] = (((audio_data[i*2] + 0x80000000) >> 24 ) & 0x000000ff);
					}
				} else {
					//this is signed so convert to unsigned 8 bit
					for (int i = 0; i < FRAMES_IN_PACKET ; i++)
					{
						mono_data[i] = (((audio_data[i] + 0x80000000) >> 24 ) & 0x000000ff);
					}
				}
			}
		}
		
		if (!paused) {
			//if its time to send more data
			QueryPerformanceCounter(&t2);
			elapsedTime = ((double)t2.QuadPart - (double)t1.QuadPart) / (double)frequency.QuadPart;
			
			if ( elapsedTime >= interval)
			{
				if (need_to_reload == 0)
				{
					//if (elapsedTime >= interval*1.5)
						//printf("WARN: over by %fs\n",elapsedTime - interval); 
					
					//new t1
					t1 = t2;
					
					//send data
					socket.Send( Address(REMOTE_IP_A,REMOTE_IP_B,REMOTE_IP_C,REMOTE_IP_D,
									REMOTE_PORT), mono_data, sizeof(mono_data) );
					interval = interval_norm;
					need_to_reload = 1;
				}
			}
			
			//see if we got a request
			Address sender;
			unsigned char buffer[256];
			int bytes_read = socket.Receive( sender, buffer, sizeof( buffer ) );
			if ( bytes_read )
			{
				count++;
				if (sender.GetA() == REMOTE_IP_A && sender.GetB() == REMOTE_IP_B && 
					sender.GetC() == REMOTE_IP_C && sender.GetD() == REMOTE_IP_D &&
					sender.GetPort() == REMOTE_PORT)
				{
					if (buffer[0] == 'S')
					{
						//if we receive an "S" increase the interval to slow down the packet rate
						interval = interval_slow;
						//printf("S\n");
						if (interval_norm > dynamic_min && interval_norm < dynamic_max)
							interval_norm += dynamic_step;
						
					}
					else if (buffer[0] == 'F')
					{
						//if we receive an "F" decease the interval to slow down the packet rate
						interval = interval_fast;
						//printf("F\n");
						if (interval_norm > dynamic_min && interval_norm < dynamic_max)
							interval_norm -= dynamic_step;
					}
				}
			}
		}
	}
	}
	
	// shutdown socket layer
	ShutdownSockets();
	
	if (  sf_close(sound_file) != 0)
	{
		printf("problem closing the wave file.\n");
	}
	printf("count: %d\n", count);
	SetColor(15);
	printf("Goodbye. Have nice day\n");
	int k;
	cin  >> k;
	getchar();
	return 0;
}


bool pressedPlayPause(void)
{
	//Look for media pause press to pause and play
	curr_space = GetAsyncKeyState(VK_MEDIA_PLAY_PAUSE);

	//check to see if program should be paused
	if (prev_space != curr_space){
		if (curr_space) {
			return true;
		}
		prev_space = curr_space;
	}
	return false;
}

//if media next pressed go to next song
bool pressedNext(void)
{
	curr_n = GetAsyncKeyState(VK_MEDIA_NEXT_TRACK);
	
	//check to see if program should be paused
	if (prev_n != curr_n){
		if (curr_n) {
			return true;
		}
		prev_n = curr_n;
	}
	return false;
}

//if media previous pressed go to previous song.
bool pressedPrevious(void)
{
	curr_p = GetAsyncKeyState(VK_MEDIA_PREV_TRACK);
		//check to see if program should be paused
	if (prev_p != curr_p){
		if (curr_p) {
			return true;
		}
		prev_p = curr_p;
	}
	return false;
}

bool isMonoOrStereo(const char * path)
{
	//open sound file
	SF_INFO sfinfo;
	sfinfo.format = 0;

	int mode = SFM_READ;
	
	SNDFILE* sound_file = sf_open(path,  mode,  &sfinfo);
	if (sound_file == NULL)
	{
		printf("Error opening sound file\n");
		return false;
	}
	
	//do some math
	int channels = sfinfo.channels;

	if ( channels == 1 || channels == 2 )
	{
		return true;
	}
	
	return false;
}

SNDFILE* getSoundfile(const char * path)
{
	//open sound file
	SF_INFO sfinfo;
	sfinfo.format = 0;
	
	//open file
	int mode = SFM_READ;
	SNDFILE* sndfile = sf_open(path,  mode,  &sfinfo);
	frames_per_second = sfinfo.samplerate;
	channels = sfinfo.channels;
	format = sfinfo.format;
	
	desired_num_reads = FRAMES_IN_PACKET;
	
	//set intervals
	interval_norm = ((double)FRAMES_IN_PACKET/(double)channels) / (double)frames_per_second - INTERVAL_SUB;
	//printf("interval norm: %f\n",interval_norm);
	interval_slow = SLOW_MUL*interval_norm;
	interval_fast = FAST_MUL*interval_norm;
	interval = interval_norm;
	dynamic_step = interval_norm * TUNE_PERC;
	dynamic_max = interval_norm + interval_norm * TUNE_MAX_DIF;
	dynamic_min = interval_norm - interval_norm * TUNE_MIN_DIF;
	
	return sndfile;
}

void SetColor(int ForgC)
{
     WORD wColor; 
     //We will need this handle to get the current background attribute
     HANDLE hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);
     CONSOLE_SCREEN_BUFFER_INFO csbi;
     
     //We use csbi for the wAttributes word.
     if(GetConsoleScreenBufferInfo(hStdOut, &csbi))
     {
        //Mask out all but the background attribute, and add in the forgournd color
          wColor = (csbi.wAttributes & 0xF0) + (ForgC & 0x0F);
          SetConsoleTextAttribute(hStdOut, wColor);     
     }
     return;
}

